@@ -7,6 +7,7 @@ from trytond.model import ModelView, fields
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
 from trytond.wizard import Wizard, StateView, StateAction, Button
+from trytond.tools import grouped_slice
 
 __all__ = ['Configuration', 'Production',
     'CreateProductionRequestStart', 'CreateProductionRequest']
@@ -27,12 +28,15 @@ class Production:
     __name__ = 'production'
 
     @classmethod
-    def generate_requests(cls, clean=True):
+    def generate_requests(cls, clean=True, warehouses=None):
         """
         For each product compute the production request that must be created
         today to meet product outputs.
 
         If clean is set, it will remove all previous requests.
+
+        If warehouses is specified it will compute the production requests
+        only for the selected warehouses.
         """
         pool = Pool()
         OrderPoint = pool.get('stock.order_point')
@@ -48,10 +52,11 @@ class Production:
                     ])
             cls.delete(reqs)
 
-        # fetch warehouse
-        warehouses = Location.search([
-                ('type', '=', 'warehouse'),
-                ])
+        if warehouses is None:
+            # fetch warehouse
+            warehouses = Location.search([
+                    ('type', '=', 'warehouse'),
+                    ])
         warehouse_ids = [w.id for w in warehouses]
         # fetch order points
         order_points = OrderPoint.search([
@@ -71,19 +76,19 @@ class Production:
                 ('purchasable', '=', False),
                 ])
         # compute requests
-        cursor = Transaction().cursor
         today = Date.today()
         requests = []
-        for i in range(0, len(products), cursor.IN_MAX):
-            product_ids = [p.id for p in products[i:i + cursor.IN_MAX]]
+        for sub_products in grouped_slice(products):
+            sub_products = list(sub_products)
+            product_ids = [p.id for p in sub_products]
             with Transaction().set_context(forecast=True,
                     stock_date_end=today):
                 pbl = Product.products_by_location(warehouse_ids,
                     product_ids, with_childs=True)
 
             # order product by supply period
-            products_period = sorted([(p.get_supply_period(), p)
-                    for p in products[i:i + cursor.IN_MAX]])
+            products_period = sorted((p.get_supply_period(), p)
+                for p in sub_products)
 
             for warehouse in warehouses:
                 quantities = dict((x, pbl.pop((warehouse.id, x), 0))
@@ -91,7 +96,7 @@ class Production:
                 shortages = cls.get_shortage(warehouse.id, product_ids, today,
                     quantities, products_period, product2ops)
 
-                for product in products[i:i + cursor.IN_MAX]:
+                for product in sub_products:
                     for date, quantity in shortages[product.id]:
                         req = cls.compute_request(product, warehouse,
                             quantity, date, company)
@@ -99,7 +104,7 @@ class Production:
                         req.set_moves()
                         requests.append(req)
         if requests:
-            cls.generate_requests(clean=False)
+            cls.generate_requests(False, warehouses)
         return requests
 
     @classmethod
@@ -172,15 +177,6 @@ class Production:
                     shortages[product_id].append((current_date, quantity))
                     current_qties[product_id] += quantity
 
-            # Update current quantities
-            with Transaction().set_context(stock_date_start=current_date,
-                    stock_date_end=current_date):
-                pbl = Product.products_by_location([location_id],
-                    product_ids, with_childs=True)
-            for key, qty in pbl.iteritems():
-                _, product_id = key
-                current_qties[product_id] += qty
-
             # Remove product with smaller period
             while (products_period
                     and products_period[0][0] <= (current_date - date).days):
@@ -188,6 +184,16 @@ class Production:
                 product_ids.remove(product.id)
             current_date += datetime.timedelta(1)
 
+            # Update current quantities with next moves
+            with Transaction().set_context(forecast=True,
+                    stock_date_start=current_date,
+                    stock_date_end=current_date):
+                pbl = Product.products_by_location([location_id],
+                    product_ids, with_childs=True)
+            for key, qty in pbl.iteritems():
+                _, product_id = key
+                current_qties[product_id] += qty
+
         return shortages
 
 
@@ -206,10 +212,14 @@ class CreateProductionRequest(Wizard):
             ])
     create_ = StateAction('stock_supply_production.act_production_request')
 
+    @property
+    def _requests_parameters(self):
+        return {}
+
     def do_create_(self, action):
         pool = Pool()
         Production = pool.get('production')
-        Production.generate_requests()
+        Production.generate_requests(**self._requests_parameters)
         return action, {}
 
     def transition_create_(self):
